# -*- coding: utf-8 -*-
from collective.taxonomy.interfaces import ITaxonomy
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from transaction import commit
from zope.component import getUtility
from zope.component import queryUtility
from zope.schema.interfaces import IVocabularyFactory

import json
import unittest


class TestControlpanelVocabularies(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_INTEGRATION_TESTING

    def setUp(self):
        """Custom shared utility setup for tests."""
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        # default values are set in italian
        self.request["LANGUAGE"] = "it"

    def set_value_for_language(self, field, data):
        values = api.portal.get_registry_record(
            field, interface=IDesignPloneSettings, default=[]
        )
        json_value = json.loads(values)
        json_value.update(data)

        api.portal.set_registry_record(
            field, json.dumps(json_value), interface=IDesignPloneSettings
        )
        commit()

    # vocabulary design.plone.vocabularies.tipologie_notizia => collective.taxonomy.tipologia_notizia  # noqa
    def test_tipologia_notizia_vocab(self):
        # factory = getUtility(
        #     IVocabularyFactory, "design.plone.vocabularies.tipologie_notizia"
        # )
        # vocab = factory(self.portal)
        taxonomy = queryUtility(ITaxonomy, name="collective.taxonomy.tipologia_notizia")
        vocab = taxonomy.makeVocabulary("it")
        self.assertEqual(
            ["Notizia", "Comunicato (stampa)", "Avviso"],
            [self.portal.translate(x.title) for x in vocab],
        )

    # TODO: en
    # def test_tipologia_notizia_vocab_in_another_language(self):
    #     self.request["LANGUAGE"] = "en"
    #     factory = getUtility(
    #         IVocabularyFactory, "design.plone.vocabularies.tipologie_notizia"
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         [],
    #         [(x.value) for x in vocab],
    #     )
    #     #  set values also for en
    #     self.set_value_for_language(
    #         field="tipologie_notizia", data={"en": ["news-foo", "news-bar"]}
    #     )
    #     factory = getUtility(
    #         IVocabularyFactory, "design.plone.vocabularies.tipologie_notizia"
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         ["", "news-foo", "news-bar"],
    #         [(x.value) for x in vocab],
    #     )

    # TODO
    # def test_tipologie_unita_organizzativa_vocab(self):
    #     factory = getUtility(
    #         IVocabularyFactory,
    #         "design.plone.vocabularies.tipologie_unita_organizzativa",
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         ["", "Politica", "Amministrativa", "Altro"],
    #         [(x.value) for x in vocab],
    #     )

    # def test_tipologia_unita_organizzativa_vocab_in_another_language(self):
    #     self.request["LANGUAGE"] = "en"
    #     factory = getUtility(
    #         IVocabularyFactory,
    #         "design.plone.vocabularies.tipologie_unita_organizzativa",
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         [],
    #         [(x.value) for x in vocab],
    #     )

    #     #  set values also for en
    #     self.set_value_for_language(
    #         field="tipologie_unita_organizzativa",
    #         data={"en": ["uo-foo", "uo-bar"]},
    #     )

    #     factory = getUtility(
    #         IVocabularyFactory,
    #         "design.plone.vocabularies.tipologie_unita_organizzativa",
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         ["", "uo-foo", "uo-bar"],
    #         [(x.value) for x in vocab],
    #     )

    def test_tipologie_tipologie_documento_vocab(self):
        # factory = getUtility(
        #     IVocabularyFactory,
        #     "design.plone.vocabularies.tipologie_documento",
        # )
        # vocab = factory(self.portal)
        taxonomy = queryUtility(
            ITaxonomy, name="collective.taxonomy.tipologia_documento"
        )
        vocab = taxonomy.makeVocabulary("it")
        self.assertEqual(
            [
                "Documenti albo pretorio",
                "Modulistica",
                "Documento funzionamento interno",
                "Atto normativo",
                "Accordo tra enti",
                "Documento attività politica",
                "Documento (tecnico) di supporto",
                "Istanza",
                "Dataset",
            ],
            [self.portal.translate(x.title) for x in vocab],
        )

    # def test_tipologie_documento_vocab_in_another_language(self):
    #     self.request["LANGUAGE"] = "en"
    #     factory = getUtility(
    #         IVocabularyFactory,
    #         "design.plone.vocabularies.tipologie_documento",
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         [],
    #         [(x.value) for x in vocab],
    #     )

    #     #  set values also for en
    #     self.set_value_for_language(
    #         field="tipologie_documento",
    #         data={"en": ["doc-foo", "doc-bar"]},
    #     )

    #     factory = getUtility(
    #         IVocabularyFactory, "design.plone.vocabularies.tipologie_documento"
    #     )
    #     vocab = factory(self.portal)
    #     self.assertEqual(
    #         ["", "doc-foo", "doc-bar"],
    #         [(x.value) for x in vocab],
    #     )

    def test_dimensioni_immagini(self):
        factory = getUtility(
            IVocabularyFactory,
            "design.plone.vocabularies.leadimage_dimension",
        )
        vocab = factory(self.portal)
        terms = {x.token: x.title for x in vocab}
        self.assertTrue("News Item" in terms)
        self.assertTrue(terms["News Item"] == "1920x600")
        self.assertTrue("UnitaOrganizzativa" in terms)
        self.assertTrue(terms["UnitaOrganizzativa"] == "1920x600")



# -*- coding: utf-8 -*-
from design.plone.contenttypes import _
from plone.dexterity.interfaces import IDexterityContent
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VocabItem(object):
    def __init__(self, token, value):
        self.token = token
        self.value = value


TAGS_MAPPING = [
    ("accesso_all_informazione", _("Accesso all'informazione")),
    ("acqua", _("Acqua")),
    ("agricoltura", _("Agricoltura")),
    ("animale_domestico", _("Animale domestico")),
    ("aria", _("Aria")),
    ("assistenza_agli_anziani", _("Assistenza agli invalidi")),
    ("assistenza_sociale", _("Assistenza sociale")),
    ("associazioni", _("Associazioni")),
    ("bilancio", _("Bilancio")),
    ("commercio_all_ingresso", _("Commercio all'ingrosso")),
    ("commercio_al_minuto", _("Commercio al minuto")),
    ("commercio_ambulante", _("Commercio ambulante")),
    ("comunicazione_istituzionale", _("Comunicazione istituzionale")),
    ("comunicazione_politica", _("Comunicazione politica")),
    ("concordi", _("Concorsi")),
    ("covid_19", _("Covid - 19")),
    ("elezioni", _("Elezioni")),
    ("energie_rinnovabili", _("Energie rinnovabili")),
    ("estero", _("Estero")),
    ("foreste", _("Foreste")),
    ("formazione_professionale", _("Formazione professionale")),
    ("gemellaggi", _("Gemellaggi")),
    ("gestione_rifiuti", _("Gestione rifiuti")),
    ("giustizia", _("Giustizia")),
    ("igiene_pubblica", _("Igiene pubblica")),
    ("immigrazione", _("Immigrazione")),
    ("imposte", _("Imposte")),
    ("imprese", _("Imprese")),
    ("inquinamento", _("Inquinamento")),
    ("integrazione_sociale", _("Integrazione sociale")),
    ("isolamento_termico", _("Isolamento termico")),
    ("istruzione", _("Istruzione")),
    ("lavoro", _("Lavoro")),
    ("matrimonio", _("Matrimonio")),
    ("mercato", _("Mercato")),
    ("mobilita_sostenibile", _("Mobilità sostenibile")),
    ("morte", _("Morte")),
    ("nascita", _("Nascita")),
    ("parcheggi", _("Parcheggi")),
    ("patrimonio_culturale", _("Patrimonio culturale")),
    ("pesca", _("Pesca")),
    ("piano_di_sviluppo", _("Piano di sviluppo")),
    ("pista_ciclabile", _("Pista ciclabile")),
    ("politica_commerciale", _("Politica commerciale")),
    ("polizia", _("Polizia")),
    ("prodotti_alimentari", _("Prodotti alimentari")),
    ("protezione_civile", _("Protezione civile")),
    ("residenza", _("Residenza")),
    ("risposta_alle_emergenze", _("Risposta alle emergenze")),
    ("sistema_giuridico", _("Sistema giuridico")),
    ("spazio_verde", _("Spazio Verde")),
    ("sport", _("Sport")),
    ("sviluppo_sostenibile", _("Sviluppo sostenibile")),
    ("tassa_sui_servizi", _("Tassa sui servizi")),
    ("tempo_libero", _("Tempo libero")),
    ("trasparenza_amministrativa", _("Trasparenza amministrativa")),
    ("trasporto_pubblico", _("Trasporto pubblico")),
    ("turismo", _("Turismo")),
    ("urbanizzazione", _("Urbanizzazione")),
    ("viaggi", _("Viaggi")),
    ("zone_pedonali", _("Zone pedonali")),
    ("ztl", _("ZTL")),
]


@implementer(IVocabularyFactory)
class TagsVocabulary(object):
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.
        items = [VocabItem(token=token, value=value) for token, value in TAGS_MAPPING]

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]

        # create a list of SimpleTerm items:
        terms = []
        for item in items:
            terms.append(
                SimpleTerm(value=item.token, token=str(item.token), title=item.value)
            )
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


TagsVocabularyFactory = TagsVocabulary()



# -*- coding: utf-8 -*-
from plone import api


def modify_incarico(obj, event):
    """
    Se cambio il titolo dell'incarico devo verificare se l'incarico è quello che
    viene puntato dalla persona. Nel caso sia quello, devo reindicizzare il
    ruolo della persona perché potrebbe essere cambiato.
    """
    idxs = ["ruolo"]
    persona = api.relation.get(target=obj, relationship="incarichi_persona")
    if not persona:
        return
    if not persona[0].isBroken():
        persona[0].from_object.reindexObject(idxs=idxs)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.restapi.testing import RelativeSession

import unittest


class SettingsControlpanelTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

        setRoles(self.portal, TEST_USER_ID, ["Manager"])

    def test_controlpanel_exists(self):
        response = self.api_session.get("/@controlpanels/design-plone-settings")

        self.assertEqual(response.status_code, 200)
        self.assertEqual(response.headers.get("Content-Type"), "application/json")

    def test_controlpanel_listed(self):
        response = self.api_session.get("/@controlpanels")

        titles = [x.get("title") for x in response.json()]
        self.assertIn("Impostazioni Design Plone", titles)



# -*- coding: utf-8 -*-
from design.plone.contenttypes.controlpanels.settings import IDesignPloneSettings
from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.app.testing import TEST_USER_ID
from plone.dexterity.utils import createContentInContainer
from plone.restapi.testing import RelativeSession
from transaction import commit

import unittest


class CustomNavigationTest(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.portal_url = self.portal.absolute_url()
        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.folder = createContentInContainer(
            self.portal, "Folder", id="folder", title="Some Folder"
        )
        self.folder2 = createContentInContainer(
            self.portal, "Folder", id="folder2", title="Some Folder 2"
        )
        self.subfolder1 = createContentInContainer(
            self.folder, "Folder", id="subfolder1", title="SubFolder 1"
        )
        self.subfolder2 = createContentInContainer(
            self.folder, "Folder", id="subfolder2", title="SubFolder 2"
        )
        self.thirdlevelfolder = createContentInContainer(
            self.subfolder1,
            "Folder",
            id="thirdlevelfolder",
            title="Third Level Folder",
        )
        self.fourthlevelfolder = createContentInContainer(
            self.thirdlevelfolder,
            "Folder",
            id="fourthlevelfolder",
            title="Fourth Level Folder",
        )
        createContentInContainer(self.folder, "Document", id="doc1", title="A document")
        commit()

    def tearDown(self):
        self.api_session.close()

    def test_return_show_in_footer_info_based_on_registry(self):
        # by default is True
        response = self.api_session.get(
            "/@navigation", params={"expand.navigation.depth": 2}
        ).json()

        self.assertIn("show_in_footer", response)
        self.assertTrue(response["show_in_footer"])

        # change it
        api.portal.set_registry_record(
            "show_dynamic_folders_in_footer",
            False,
            interface=IDesignPloneSettings,
        )
        commit()

        response = self.api_session.get(
            "/@navigation", params={"expand.navigation.depth": 2}
        ).json()

        self.assertIn("show_in_footer", response)
        self.assertFalse(response["show_in_footer"])



# -*- coding: utf-8 -*-
from plone.app.contenttypes.interfaces import INewsItem
from plone.indexer.decorator import indexer


@indexer(INewsItem)
def news_people(context, **kw):
    people = context.a_cura_di_persone
    return [persona.UID() for persona in filter(bool, [x.to_object for x in people])]


@indexer(INewsItem)
def news_uo(context, **kw):
    unita_organizzative = context.a_cura_di
    return [
        unita_organizzativa.UID()
        for unita_organizzativa in filter(
            bool, [x.to_object for x in unita_organizzative]
        )
    ]


@indexer(INewsItem)
def news_service(context, **kw):
    servizi = context.servizi_correlati
    return [servizio.UID() for servizio in filter(bool, [x.to_object for x in servizi])]


@indexer(INewsItem)
def news_venue(context, **kw):
    luoghi = context.luoghi_correlati
    return [luogo.UID() for luogo in filter(bool, [x.to_object for x in luoghi])]



# -*- coding: utf-8 -*-
from plone import api
from plone.dexterity.interfaces import IDexterityContent
from zope.globalrequest import getRequest
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary


class VocabItem(object):
    def __init__(self, token, value):
        self.token = token
        self.value = value


@implementer(IVocabularyFactory)
class AvailableServicesVocabulary(object):
    """ """

    def __call__(self, context):
        # Just an example list of content for our vocabulary,
        # this can be any static or dynamic data, a catalog result for example.

        portal_catalog = api.portal.get_tool("portal_catalog")
        results = portal_catalog.searchResults(portal_type="Servizio")

        # Fix context if you are using the vocabulary in DataGridField.
        # See https://github.com/collective/collective.z3cform.datagridfield/issues/31:  # NOQA: 501
        if not IDexterityContent.providedBy(context):
            req = getRequest()
            context = req.PARENTS[0]
        # create a list of SimpleTerm items:
        terms = []
        for result in results:
            brain = result.getObject()
            uid = brain.UID()
            terms.append(SimpleTerm(value=uid, token=uid, title=brain.title))
        # Create a SimpleVocabulary from the terms list and return it:
        return SimpleVocabulary(terms)


AvailableServicesVocabularyFactory = AvailableServicesVocabulary()



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.servizio import IServizio
from design.plone.contenttypes.restapi.serializers.summary import (
    DefaultJSONSummarySerializer,
)
from design.plone.contenttypes.restapi.serializers.summary import (
    get_taxonomy_information,
)
from plone.dexterity.utils import iterSchemata
from plone.restapi.interfaces import IFieldSerializer
from plone.restapi.interfaces import ISerializeToJsonSummary
from zope.component import adapter
from zope.component import queryMultiAdapter
from zope.interface import implementer
from zope.interface import Interface
from zope.schema import getFields


@implementer(ISerializeToJsonSummary)
@adapter(IServizio, Interface)
class SerializeServizioToJsonSummary(DefaultJSONSummarySerializer):
    def __call__(self, **kwargs):
        summary = super().__call__(**kwargs)
        fields = ["canale_digitale"]
        for schema in iterSchemata(self.context):
            for name, field in getFields(schema).items():
                if name not in fields:
                    continue

                # serialize the field
                serializer = queryMultiAdapter(
                    (field, self.context, self.request), IFieldSerializer
                )
                value = serializer()
                summary[name] = value

        parent = self.context.aq_inner.aq_parent
        summary["parent_title"] = parent.title
        summary["parent_url"] = parent.absolute_url()
        get_taxonomy_information("person_life_events", self.context, summary)
        get_taxonomy_information("business_events", self.context, summary)
        return summary



# -*- coding: utf-8 -*-
from design.plone.contenttypes.interfaces.pagina_argomento import IPaginaArgomento
from plone.dexterity.content import Container
from zope.interface import implementer


@implementer(IPaginaArgomento)
class PaginaArgomento(Container):
    """ """



# -*- coding: utf-8 -*-

from DateTime import DateTime
from design.plone.contenttypes.utils import text_in_block
from openpyxl import Workbook
from openpyxl.cell.cell import ILLEGAL_CHARACTERS_RE
from openpyxl.styles import Alignment
from openpyxl.styles import Font
from openpyxl.styles import PatternFill
from openpyxl.utils import get_column_letter

from plone import api
from Products.Five import BrowserView
import io


FLAG = '<i class="fa-solid fa-check"></i>'


class CheckServizi(BrowserView):
    cds = None

    def is_anonymous(self):
        return api.user.is_anonymous()

    def get_canale_accesso_info(self, servizio):
        canale_fisico = getattr(servizio, "canale_fisico", None)
        canale_digitale = text_in_block(getattr(servizio, "canale_digitale", None))
        canale_digitale_link = getattr(servizio, "canale_digitale_link", None)
        if canale_digitale and canale_digitale_link and canale_fisico:
            return "D e F"
        elif canale_digitale and canale_digitale_link:
            return "D"
        elif canale_fisico:
            return "F"
        # elif canale_digitale or canale_digitale_link:
        #     return "&frac12;D"
        return None

    def information_dict(self, servizio):
        return {
            "title": getattr(servizio, "title"),
            "description": getattr(servizio, "description", None),
            "condizioni_di_servizio": getattr(servizio, "condizioni_di_servizio", None),
            "tassonomia_argomenti": getattr(servizio, "tassonomia_argomenti", None),
            "a_chi_si_rivolge": text_in_block(
                getattr(servizio, "a_chi_si_rivolge", None)
            ),
            "come_si_fa": text_in_block(getattr(servizio, "come_si_fa", None)),
            "cosa_si_ottiene": text_in_block(
                getattr(servizio, "cosa_si_ottiene", None)
            ),
            "canale_accesso": self.get_canale_accesso_info(servizio),
            "cosa_serve": text_in_block(getattr(servizio, "cosa_serve", None)),
            "tempi_e_scadenze": text_in_block(
                getattr(servizio, "tempi_e_scadenze", None)
            ),
            "ufficio_responsabile": getattr(servizio, "ufficio_responsabile", None),
            "contact_info": getattr(servizio, "contact_info", None),
        }

    def plone2volto(self, url):
        portal_url = api.portal.get().absolute_url()
        frontend_domain = api.portal.get_registry_record(
            "volto.frontend_domain", default=""
        )
        if frontend_domain and url.startswith(portal_url):
            return url.replace(portal_url, frontend_domain, 1)
        return url

    def get_relation_title(self, information_dict, label):
        result = ""
        for item in information_dict[label]:
            if not item:
                continue
            if not item.to_object:
                continue
            result = result + " " + item.to_object.title
        return result

    def get_servizi(self, full_report=False):
        if self.is_anonymous():
            return []
        pc = api.portal.get_tool("portal_catalog")

        # show_inactive ha sempre avuto una gestione... particolare! aggiungo ai
        # kw effectiveRange = DateTime() che è quello che fa Products.CMFPlone
        # nel CatalogTool.py
        query = {
            "portal_type": "Servizio",
            "review_state": "published",
            # "show_inactive": False,
        }
        brains = pc(query, **{"effectiveRange": DateTime()})
        results = {}
        for brain in brains:
            servizio = brain.getObject()
            # if safe_hasattr(servizio, "condizioni_di_servizio") and getattr(
            #     servizio, "condizioni_di_servizio"
            # ):
            #     continue

            # Se chiediamo di vedere anche le condizioni di servizio, lo teniamo
            self.cds = self.request.get("condizioni_di_servizio", None)
            information_dict = self.information_dict(servizio)
            if not self.cds:
                del information_dict["condizioni_di_servizio"]

            if all(information_dict.values()):
                continue

            parent = servizio.aq_inner.aq_parent
            if parent.title not in results:
                results[parent.title] = {
                    "url": self.plone2volto(parent.absolute_url()),
                    "children": [],
                }

            tassonomia_argomenti = self.get_relation_title(
                information_dict, "tassonomia_argomenti"
            )
            ufficio_responsabile = self.get_relation_title(
                information_dict, "ufficio_responsabile"
            )

            contatti = self.get_relation_title(information_dict, "contact_info")

            if full_report:
                results[parent.title]["children"].append(
                    {
                        "title": servizio.title,
                        "url": self.plone2volto(servizio.absolute_url()),
                        "data": {
                            "title": information_dict.get("title") or "",
                            "description": information_dict.get("description") or "",
                            "condizioni_di_servizio": information_dict.get(
                                "condizioni_di_servizio"
                            )
                            or "",
                            "tassonomia_argomenti": tassonomia_argomenti or "",
                            "a_chi_si_rivolge": information_dict.get("a_chi_si_rivolge")
                            or "",
                            "come_si_fa": information_dict.get("come_si_fa") or "",
                            "cosa_si_ottiene": information_dict.get("cosa_si_ottiene")
                            or "",
                            "canale_accesso": information_dict.get("canale_accesso")
                            or "",
                            "cosa_serve": information_dict.get("cosa_serve") or "",
                            "tempi_e_scadenze": information_dict.get("tempi_e_scadenze")
                            or "",
                            "ufficio_responsabile": ufficio_responsabile or "",
                            "contact_info": contatti or "",
                        },
                    }
                )
            else:
                results[parent.title]["children"].append(
                    {
                        "title": servizio.title,
                        "url": self.plone2volto(servizio.absolute_url()),
                        "data": {
                            "title": information_dict.get("title") and FLAG or "",
                            "description": information_dict.get("description")
                            and FLAG
                            or "",
                            "condizioni_di_servizio": information_dict.get(
                                "condizioni_di_servizio"
                            )
                            and FLAG
                            or "",
                            "tassonomia_argomenti": tassonomia_argomenti and FLAG or "",
                            "a_chi_si_rivolge": information_dict.get("a_chi_si_rivolge")
                            and FLAG
                            or "",
                            "come_si_fa": information_dict.get("come_si_fa")
                            and FLAG
                            or "",
                            "cosa_si_ottiene": information_dict.get("cosa_si_ottiene")
                            and FLAG
                            or "",
                            "canale_accesso": information_dict.get("canale_accesso")
                            or "",
                            "cosa_serve": information_dict.get("cosa_serve")
                            and FLAG
                            or "",
                            "tempi_e_scadenze": information_dict.get("tempi_e_scadenze")
                            and FLAG
                            or "",
                            "ufficio_responsabile": ufficio_responsabile and FLAG or "",
                            "contact_info": contatti and FLAG or "",
                        },
                    }
                )

        results = dict(sorted(results.items()))
        for key in results:
            results[key]["children"].sort(key=lambda x: x["title"])
        return results


class DownloadCheckServizi(CheckServizi):
    CT = "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet"

    def __call__(self):
        HEADER = [
            "Titolo",
            "Descrizione",
            "Argomenti",
            "A chi è rivolto",
            "Come fare per",
            "Cosa si ottiene",
            "Canale di accesso",
            "Cosa serve",
            "Tempi e scadenze",
            "Unità org. responsabile",
            "Contatti",
        ]

        cds = self.request.form.get("condizioni_di_servizio", None)
        if cds:
            EMPTY_ROW = [""] * 12
            HEADER.insert(2, "Condizioni di servizio")
        else:
            EMPTY_ROW = [""] * 11

        full_report = self.request.form.get("full", False)
        servizi = self.get_servizi(full_report)

        data = []
        for category in servizi:
            data.append([category] + [""] * 10 + [servizi[category]["url"]])
            data.append(HEADER)

            if full_report:
                for servizio in servizi[category]["children"]:
                    dati_servizio = [
                        servizio["title"],
                        servizio["data"]["description"] or "",
                        servizio["data"]["tassonomia_argomenti"] or "",
                        servizio["data"]["a_chi_si_rivolge"] or "",
                        servizio["data"]["come_si_fa"] or "",
                        servizio["data"]["cosa_si_ottiene"] or "",
                        servizio["data"]["canale_accesso"] or "",
                        servizio["data"]["cosa_serve"] or "",
                        servizio["data"]["tempi_e_scadenze"] or "",
                        servizio["data"]["ufficio_responsabile"] or "",
                        servizio["data"]["contact_info"] or "",
                        servizio["url"],
                    ]
                    if cds:
                        condizioni_di_servizio = (
                            servizio["data"]["condizioni_di_servizio"] or ""
                        )
                        dati_servizio.insert(2, condizioni_di_servizio)
                    data.append(dati_servizio)
            else:
                for servizio in servizi[category]["children"]:
                    dati_servizio = [
                        servizio["title"],
                        servizio["data"]["description"] and "V" or "",
                        servizio["data"]["tassonomia_argomenti"] and "V" or "",
                        servizio["data"]["a_chi_si_rivolge"] and "V" or "",
                        servizio["data"]["come_si_fa"] and "V" or "",
                        servizio["data"]["cosa_si_ottiene"] and "V" or "",
                        servizio["data"]["canale_accesso"] and "V" or "",
                        servizio["data"]["cosa_serve"] and "V" or "",
                        servizio["data"]["tempi_e_scadenze"] and "V" or "",
                        servizio["data"]["ufficio_responsabile"] and "V" or "",
                        servizio["data"]["contact_info"] and "V" or "",
                        servizio["url"],
                    ]
                    if cds:
                        condizioni_di_servizio = (
                            servizio["data"]["condizioni_di_servizio"] or ""
                        )
                        dati_servizio.insert(2, condizioni_di_servizio)
                    data.append(dati_servizio)
            data.append(EMPTY_ROW)
            data.append(EMPTY_ROW)

        workbook = Workbook()
        sheet = workbook.active
        sheet.title = "Servizi"
        link_font = Font(underline="single", color="0563C1")
        link_fill = PatternFill(fill_type="solid", fgColor="DDDDDD")
        alignment = Alignment(horizontal="center", vertical="top")

        for i, row in enumerate(data, start=1):
            have_url = row[0] != "Titolo" and bool(row[0])
            if have_url:
                url = row.pop()

            sheet.append([ILLEGAL_CHARACTERS_RE.sub(r" ", x) for x in row])

            if row[0] == "Titolo":
                for index, cell in enumerate(sheet[i]):
                    cell.fill = link_fill
                    if index != 0:
                        column_letter = get_column_letter(cell.column)
                        sheet.column_dimensions[column_letter].width = 20

            if have_url:
                for index, cell in enumerate(sheet[i]):
                    if index == 0:
                        cell = sheet.cell(row=i, column=1)
                        cell.font = link_font
                        cell.alignment = cell.alignment.copy(wrapText=True)
                        cell.hyperlink = url
                        column_letter = get_column_letter(cell.column)
                        sheet.column_dimensions[column_letter].width = 40
                    else:
                        cell.alignment = alignment

        bytes_io = io.BytesIO()
        workbook.save(bytes_io)
        data = bytes_io.getvalue()
        self.request.response.setHeader("Content-Length", len(data))
        self.request.RESPONSE.setHeader("Content-Type", self.CT)
        self.request.response.setHeader(
            "Content-Disposition",
            "attachment; filename=check_servizi.xlsx",
        )
        return data



# -*- coding: utf-8 -*-

from design.plone.contenttypes.testing import (
    DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING,
)
from plone import api
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import SITE_OWNER_NAME
from plone.app.testing import SITE_OWNER_PASSWORD
from plone.restapi.testing import RelativeSession

import unittest


class TestFolderSchema(unittest.TestCase):
    layer = DESIGN_PLONE_CONTENTTYPES_API_FUNCTIONAL_TESTING

    def setUp(self):
        self.app = self.layer["app"]
        self.portal = self.layer["portal"]
        self.request = self.layer["request"]
        self.portal_url = self.portal.absolute_url()
        setRoles(self.portal, TEST_USER_ID, ["Manager"])

        self.api_session = RelativeSession(self.portal_url)
        self.api_session.headers.update({"Accept": "application/json"})
        self.api_session.auth = (SITE_OWNER_NAME, SITE_OWNER_PASSWORD)

    def tearDown(self):
        self.api_session.close()

    def test_behaviors_enabled_for_folder(self):
        portal_types = api.portal.get_tool(name="portal_types")
        self.assertEqual(
            portal_types["Folder"].behaviors,
            (
                "plone.dublincore",
                "plone.namefromtitle",
                "plone.allowdiscussion",
                "plone.excludefromnavigation",
                "plone.shortname",
                "plone.constraintypes",
                "plone.relateditems",
                "plone.nextprevioustoggle",
                "design.plone.contenttypes.behavior.exclude_from_search",
            ),
        )
